"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.githubDownload = void 0;
const fse = require("fs-extra");
const url = require("url");
const axios = require("axios");
const AUTHOR = 1;
const REPOSITORY = 2;
const BRANCH = 4;
let repoInfo = {};
let outputDirectory = `${process.cwd()}/`;
let localRootDirectory = "";
const basicOptions = {
    method: "get",
    responseType: "arrayBuffer",
};
function preprocessURL(repoURL) {
    const len = repoURL.length;
    if (repoURL[len - 1] === "/") {
        return repoURL.slice(0, len - 1);
    }
    return repoURL;
}
function parseInfo(repoInfo) {
    const repoURL = preprocessURL(repoInfo.url);
    const repoPath = url.parse(repoURL, true).pathname;
    const splitPath = repoPath.split("/");
    const info = {};
    info.author = splitPath[AUTHOR];
    info.repository = splitPath[REPOSITORY];
    info.branch = splitPath[BRANCH];
    info.rootName = splitPath[splitPath.length - 1];
    info.urlPrefix = `https://api.github.com/repos/${info.author}/${info.repository}/contents/`;
    info.urlPostfix = `?ref=${info.branch}`;
    if (splitPath[BRANCH]) {
        info.resPath = repoPath.substring(repoPath.indexOf(splitPath[BRANCH]) + splitPath[BRANCH].length + 1);
    }
    if (!repoInfo.fileName || repoInfo.fileName === "") {
        info.downloadFileName = info.rootName;
    }
    else {
        info.downloadFileName = repoInfo.fileName;
    }
    if (repoInfo.rootDirectory === "false") {
        info.rootDirectoryName = "";
    }
    else if (!repoInfo.rootDirectory ||
        repoInfo.rootDirectory === "" ||
        repoInfo.rootDirectory === "true") {
        info.rootDirectoryName = `${info.rootName}/`;
    }
    else {
        info.rootDirectoryName = `template/`;
    }
    if (repoInfo.to) {
        if (repoInfo.to[repoInfo.to.length - 1] !== "/")
            repoInfo.to = repoInfo.to + "/";
        outputDirectory = repoInfo.to;
    }
    return info;
}
function extractFilenameAndDirectoryFrom(path) {
    const components = path.split("/");
    const filename = components[components.length - 1];
    const directory = path.substring(0, path.length - filename.length);
    return {
        filename,
        directory,
    };
}
function downloadFile(url, pathname, logger) {
    return new Promise((resolve, reject) => {
        axios(Object.assign(Object.assign({}, basicOptions), { responseType: "stream", url }))
            .then((response) => __awaiter(this, void 0, void 0, function* () {
            yield fse.mkdirp(pathname.directory);
            const localPathname = pathname.directory + pathname.filename;
            response.data
                .pipe(fse.createWriteStream(localPathname))
                .on("error", (error) => {
                console.log("ERROR", error);
            })
                .on("close", () => {
                logger.log(`Downloading: ${pathname.filename}`);
                resolve(true);
            });
        }))
            .catch((error) => {
            console.log("Error downloading file: ", pathname.filename);
            reject(false);
        });
    });
}
function removeResPathFrom(path) {
    return path.substring(decodeURI(repoInfo.resPath).length + 1);
}
function constructLocalPathname(repoPath) {
    const partialPath = extractFilenameAndDirectoryFrom(removeResPathFrom(repoPath));
    localRootDirectory = outputDirectory + repoInfo.rootDirectoryName;
    const localDirectory = localRootDirectory + partialPath.directory;
    return {
        filename: partialPath.filename,
        directory: localDirectory,
    };
}
function processResponse(response, dirPaths, logger) {
    return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
        const { data } = response;
        const promises = [];
        for (let i = 0; i < data.length; i++) {
            if (data[i].type === "dir") {
                dirPaths.push(data[i].path);
            }
            else if (data[i].download_url) {
                const pathname = constructLocalPathname(data[i].path);
                promises.push(downloadFile(data[i].download_url, pathname, logger));
            }
            else {
                console.log("Error:", data[i]);
                reject(false);
            }
        }
        if (promises.length === 0 || Promise.all(promises))
            resolve(true);
    }));
}
function iterateDirectory(dirPaths, logger) {
    return new Promise((resolve, reject) => {
        axios(Object.assign(Object.assign({}, basicOptions), { url: repoInfo.urlPrefix + dirPaths.pop() + repoInfo.urlPostfix }))
            .then((response) => __awaiter(this, void 0, void 0, function* () {
            const finished = yield processResponse(response, dirPaths, logger);
            if (finished)
                resolve(true);
        }))
            .catch((error) => {
            console.log("\nRate limit exceeded, please try initializing again in an hour\n");
            reject(false);
        });
    });
}
function downloadDirectory(response, logger) {
    return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
        try {
            const dirPaths = [];
            dirPaths.push(repoInfo.resPath);
            yield processResponse(response, dirPaths, logger);
            while (dirPaths.length !== 0) {
                const finished = yield iterateDirectory(dirPaths, logger);
                if (finished)
                    continue;
            }
            resolve(true);
        }
        catch (err) {
            reject(false);
        }
    }));
}
function githubDownload(options, logger) {
    return new Promise((resolve, reject) => {
        repoInfo = parseInfo(options);
        axios(Object.assign(Object.assign({}, basicOptions), { url: repoInfo.urlPrefix + repoInfo.resPath + repoInfo.urlPostfix }))
            .then((response) => __awaiter(this, void 0, void 0, function* () {
            if (response.data instanceof Array) {
                const finished = yield downloadDirectory(response, logger);
                if (finished) {
                    resolve(true);
                }
            }
            else {
                const partialPath = extractFilenameAndDirectoryFrom(decodeURI(repoInfo.resPath));
                downloadFile(response.data.download_url, Object.assign(Object.assign({}, partialPath), { directory: outputDirectory }), logger);
                localRootDirectory = outputDirectory;
                resolve(true);
            }
        }))
            .catch((error) => {
            reject(false);
        });
    });
}
exports.githubDownload = githubDownload;
//# sourceMappingURL=download.js.map